/*****************************************************
 * RS422通信を使用した　ＬＥＤ　７セグ　表示器
 * 使用マイコン　PIC16F1827
 * コンパイラ　XC8 Ver1.45
 * 　2023/05/15
******************************************************/
#include <xc.h>
#include <stdio.h>
#include <stdlib.h>

/*****コンフィグレーション設定*********/
// コンフィギュレーション１の設定+

#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON      // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)

/* 内部クロック　８ＭＨｚ */
#define  _XTAL_FREQ     8000000		//ディレイ用

/********　変数定義　************/

/********　サブ関数　************/
void display(unsigned char data);    
void send_data(unsigned char data);    
void putch(unsigned char byte);

/********　メイン開始　************/
void main(void) {

	/********　変数定義　************/
    unsigned  char  RxData;  
    unsigned  char  MY_ID;
    unsigned  char  ID_NO;
    unsigned  char  DISPLAY_NU;
    
	/*******初期設定***** */
    OSCCON = 0x72;				//内臓クロック８HMz指定(BPLL Off
    ANSELA = 0x00;				//A/D停止
    ANSELB = 0x00;				//A/D停止

    TRISA  = 0b00000000;		//ポートＡすべて出力
    TRISB  = 0b11110010;		//ポートＢ　RB1シリアル入力
                                
    PORTA = 0;                  //すべてオフ
    
    OPTION_REG =0b00001111;     //  
    WPUB = 0b11110000;          //RB4～RB7　プルアップ
    nWPUEN = 0;                 //プルアップ有効
        
    /* USART初期設定*/
    //　TXCKSEL　0=RB2　1=RB5　　　RXDTSEL　0=RB1　1=RB2
    TXCKSEL = 0;                    // TX = RB2
    RXDTSEL = 0;                    // Rx = RB1
    TXSTA = 0x24;                   //シリアル送信設定
    RCSTA = 0x90;                   //シリアル受信設定
	BAUDCON=0x08;                   //
	SPBRG = 207;                    //SPBRG=207時9600bps
                                    //SPBRG=103時19200bps
 
    __delay_ms(500);
 //   printf("HELLO ! Ver.1 \r\n");	//\r　復帰　\n　改行
 //   printf("1:%02X\r\n",PORTB);
    MY_ID = ~PORTB;                 //ジャンパー設定よりＩＤを取得
//    printf("2:%02X\r\n",MY_ID);    
    MY_ID = MY_ID >>4;
//    printf("3:%02X\r\n",MY_ID);    
    display(MY_ID);
    __delay_ms(3000);
    PORTA = 0;          //すべて消す      
 //     
    while (1) {        
        if(RCIF){                           //データの受信
            RxData = RCREG;
 //           printf("4:%02X \r\n",RxData);
            ID_NO = RxData >> 4;            //コマンドからID取得
            DISPLAY_NU = RxData &0x0f ;     //表示数字の取得     
            if(MY_ID == ID_NO){             //設定IDと受信IDが一致するか
                display(DISPLAY_NU);        //一致していれば数字を表示する
            }
        } 
     }
}

/*************************************
 * 7aegを表示する
 *************************************/

void display(unsigned char data)
{
    switch(data){            
        case 0 : //printf("0");
        PORTA = 0;
        PORTA = 0b01011111;              
        break;

        case 1 : //printf("1");
        PORTA = 0;
        PORTA = 0b00001100;          
        break;
                
        case 2 : //printf("2");
        PORTA = 0;
        PORTA = 0b11010110;              
        break;
                
        case 3 : //printf("3");
        PORTA = 0;
        PORTA = 0b10011110;              
        break;
                
        case 4 : //printf("4");
        PORTA = 0;
        PORTA = 0b10001101;              
        break;
                
        case 5 : //printf("5");
        PORTA = 0;
        PORTA = 0b10011011;              
        break;

        case 6 : //printf("6");
        PORTA = 0;
        PORTA = 0b11011011;              
        break;

        case 7 : //printf("7");     
        PORTA = 0;
        PORTA = 0b00001111;              
        break;

        case 8 : //printf("8");
        PORTA = 0;
        PORTA = 0b11011111;
        break;                

        case 9 : //printf("9");
        PORTA = 0;
        PORTA = 0b10011111;          
        break;                

        case 0x0a : //printf("0x0a");
        PORTA = 0;
        PORTA = 0b11001111;          
        break;                
        
        case 0x0b : //printf("0x0b");
        PORTA = 0;
        PORTA = 0b11011001;          
        break;                

        case 0x0c : //printf("0x0c");
        PORTA = 0;
        PORTA = 0b01010011;          
        break;                
        
        case 0x0d : //printf("0x0d");
        PORTA = 0;
        PORTA = 0b11011100;          
        break;                
        
        case 0x0e : //printf("0x0e");
        PORTA = 0;
        PORTA = 0b11010011;          
        break;   
        
        case 0x0f : //printf("0x0f");
        PORTA = 0;
        PORTA = 0b11000011;          
        break;                
        
        default: //printf("-\r\n");
        PORTA = 0;
        PORTA = 0b10000000;
        break; 
    }
}
/*************************************
 * UART 1バイト送信
 *************************************/
void send_data(unsigned char data)
{
    TXREG=data;                     //1バイトのデータ送信
	while(!TXIF)continue;           //送信完了まで待つ
	TXIF=0;                         //フラグをクリア
}
 /************************************************************
 *　　　　　　　Printf用
 *
 ***********************************************************/
void putch(unsigned char byte){
		RCSTAbits.SPEN=1;
		TXREG=byte;
		while(!TXIF)continue;
		TXIF=0;
}